package boshclient

import (
	"fmt"
	"net/http"
)

func (c *Client) GetTask(taskID int) (BoshTask, error) {
	c.logger.Printf("getting task %d from bosh\n", taskID)
	var getTaskResponse BoshTask

	if err := c.getDataFromBoshCheckingForErrors(
		fmt.Sprintf("%s/tasks/%d", c.boshURL, taskID),
		http.StatusOK,
		&getTaskResponse,
	); err != nil {
		return BoshTask{}, err
	}

	return getTaskResponse, nil
}

type TaskOutput struct {
	ExitCode int    `json:"exit_code"`
	StdOut   string `json:"stdout"`
	StdErr   string `json:"stderr"`
}

func (c *Client) GetTaskOutput(taskID int) ([]TaskOutput, error) {
	c.logger.Printf("getting task output for task %d from bosh\n", taskID)
	outputs := []TaskOutput{}
	var output TaskOutput
	outputReadyCallback := func() {
		outputs = append(outputs, output)
		output = TaskOutput{}
		// `output` is reused for JSON decoding, so use a fresh struct;
		// else you will override your previous values with the current one
	}

	err := c.getMultipleDataFromBoshCheckingForErrors(
		fmt.Sprintf("%s/tasks/%d/output?type=result", c.boshURL, taskID),
		http.StatusOK,
		&output,
		outputReadyCallback,
	)

	return outputs, err
}
