package broker_test

import (
	"bytes"
	"io"
	"log"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/pivotal-cf/on-demand-service-broker-sdk/serviceadapter"
	"github.com/pivotal-cf/on-demand-service-broker/broker"
	"github.com/pivotal-cf/on-demand-service-broker/broker/fakes"

	"testing"
)

const (
	existingPlanID                      = "some-plan-id"
	existingPlanInstanceGroupName       = "instance-group-name"
	secondExistingPlanInstanceGroupName = "instance-group-name-the-second"
	serviceInstanceLimit                = 3

	secondPlanID = "another-plan"
)

var (
	b                 *broker.Broker
	brokerCreationErr error
	boshClient        *fakes.FakeBoshClient
	cfClient          *fakes.FakeCloudFoundryClient
	serviceAdapter    *fakes.FakeServiceAdapterClient
	fakeDeployer      *fakes.FakeDeployer
	serviceCatalog    broker.ServiceCatalog
	logBuffer         *bytes.Buffer

	existingPlan = broker.Plan{
		ID: existingPlanID,
		Update: &serviceadapter.Update{
			Canaries:        1,
			CanaryWatchTime: "100-200",
			UpdateWatchTime: "100-200",
			MaxInFlight:     5,
		},
		Quotas: &broker.Quotas{
			ServiceInstanceLimit: serviceInstanceLimit,
		},
		Properties: serviceadapter.Properties{
			"super": "no",
		},
		InstanceGroups: []serviceadapter.InstanceGroup{
			{
				Name:           existingPlanInstanceGroupName,
				VMType:         "vm-type",
				PersistentDisk: "disk-type",
				Instances:      42,
				Networks:       []string{"networks"},
				AZs:            []string{"my-az1", "my-az2"},
			},
			{
				Name:      secondExistingPlanInstanceGroupName,
				VMType:    "vm-type",
				Instances: 55,
				Networks:  []string{"networks2"},
			},
		},
	}
	secondPlan = broker.Plan{
		ID: secondPlanID,
		Properties: serviceadapter.Properties{
			"super": "yes",
		},
		InstanceGroups: []serviceadapter.InstanceGroup{
			{
				Name:           existingPlanInstanceGroupName,
				VMType:         "vm-type1",
				PersistentDisk: "disk-type1",
				Instances:      44,
				Networks:       []string{"networks1"},
				AZs:            []string{"my-az4", "my-az5"},
			},
		},
	}
)

var _ = BeforeEach(func() {
	boshClient = new(fakes.FakeBoshClient)
	serviceAdapter = new(fakes.FakeServiceAdapterClient)
	fakeDeployer = new(fakes.FakeDeployer)
	cfClient = new(fakes.FakeCloudFoundryClient)
	serviceCatalog = broker.ServiceCatalog{
		ID:   "service-id",
		Name: "a-cool-redis-service",
		Plans: []broker.Plan{
			existingPlan,
			secondPlan,
		},
	}

	logBuffer = new(bytes.Buffer)
})

var _ = JustBeforeEach(func() {
	b, brokerCreationErr = broker.New(
		boshClient,
		cfClient,
		serviceAdapter,
		fakeDeployer,
		serviceCatalog,
		log.New(io.MultiWriter(GinkgoWriter, logBuffer), "[broker-unit-tests] ", log.LstdFlags),
	)
})

func TestBroker(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "Broker Suite")
}

func deploymentName(instanceID string) string {
	return "service-instance_" + instanceID
}
