package broker_test

import (
	"errors"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("Initializing the broker", func() {
	Context("when there are no pre-existing instances", func() {
		BeforeEach(func() {
			cfClient.CountInstancesOfServiceOfferingReturns(map[string]int{}, nil)
		})

		It("returns no error", func() {
			Expect(brokerCreationErr).NotTo(HaveOccurred())
		})
	})

	Context("when there are no pre-existing instances of configured plans", func() {
		Context("service catalog contains the existing instance's plan", func() {
			BeforeEach(func() {
				cfClient.CountInstancesOfServiceOfferingReturns(map[string]int{existingPlanID: 1}, nil)
			})

			It("returns no error", func() {
				Expect(brokerCreationErr).NotTo(HaveOccurred())
			})
		})

		Context("service catalog does not contain a plan with zero instances", func() {
			BeforeEach(func() {
				cfClient.CountInstancesOfServiceOfferingReturns(map[string]int{existingPlanID: 1, "non-existing-plan": 0}, nil)
			})

			It("returns no error", func() {
				Expect(brokerCreationErr).NotTo(HaveOccurred())
			})
		})

		Context("service catalog does not contain the existing instance's plan", func() {
			BeforeEach(func() {
				cfClient.CountInstancesOfServiceOfferingReturns(map[string]int{"i-dont-exist-in-the-catalog": 1}, nil)
			})

			It("returns an error", func() {
				Expect(brokerCreationErr).To(MatchError(ContainSubstring("You cannot change the plan_id of a plan that has existing service instances")))
			})
		})

		Context("when instances cannot be retrieved", func() {
			BeforeEach(func() {
				cfClient.CountInstancesOfServiceOfferingReturns(nil, errors.New("flurp"))
			})

			It("returns an error", func() {
				Expect(brokerCreationErr).To(HaveOccurred())
			})
		})
	})
})
