package broker

import (
	"encoding/json"
	"fmt"

	"github.com/pivotal-cf/brokerapi"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
)

func (b *Broker) Deprovision(instanceID string, _ brokerapi.DeprovisionDetails, asyncAllowed bool) (brokerapi.DeprovisionServiceSpec, error) {
	b.deploymentLock.Lock()
	defer b.deploymentLock.Unlock()

	if !asyncAllowed {
		return brokerapi.DeprovisionServiceSpec{}, brokerapi.ErrAsyncRequired
	}

	errs := func(err error) (brokerapi.DeprovisionServiceSpec, error) {
		b.logger.Println(err)
		return brokerapi.DeprovisionServiceSpec{IsAsync: true}, convertToDisplayableError(err, b.logger)
	}

	b.logger.Printf("deleting deployment for instance %s\n", instanceID)
	taskID, err := b.boshClient.DeleteDeployment(deploymentName(instanceID))
	switch err.(type) {
	case boshclient.DeploymentNotFoundError:
		b.logger.Println(fmt.Errorf("error deprovisioning: instance %s, not found ", instanceID))
		return errs(DisplayableError{brokerapi.ErrInstanceDoesNotExist})
	case error:
		return errs(fmt.Errorf("error deprovisioning: deleting bosh deployment: %s", err))
	}

	b.logger.Printf("Bosh task id for Delete instance %s was %d\n", instanceID, taskID)

	operationData, err := json.Marshal(OperationData{OperationType: OperationTypeDelete, BoshTaskID: taskID})
	if err != nil {
		return errs(err)
	}

	return brokerapi.DeprovisionServiceSpec{IsAsync: true, OperationData: string(operationData)}, nil
}
