package broker_test

import (
	"errors"

	"github.com/pivotal-cf/on-demand-service-broker/mgmtapi"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("Instances", func() {
	Describe("listing all instances", func() {
		BeforeEach(func() {
			cfClient.GetInstancesOfServiceOfferingReturns([]string{"red", "green", "blue"}, nil)
		})

		It("returns a list of instances with ID, plan ID and deployment name", func() {
			red := mgmtapi.Instance{InstanceID: "red"}
			green := mgmtapi.Instance{InstanceID: "green"}
			blue := mgmtapi.Instance{InstanceID: "blue"}
			Expect(b.Instances()).To(ConsistOf(red, green, blue))
		})

		Context("when the list of instances cannot be retrieved", func() {
			BeforeEach(func() {
				cfClient.GetInstancesOfServiceOfferingReturns(nil, errors.New("fnuh"))
			})

			It("returns an error", func() {
				_, err := b.Instances()
				Expect(err).To(MatchError(ContainSubstring("fnuh")))
			})
		})
	})
})
