package broker

import (
	"fmt"

	"github.com/pivotal-cf/on-demand-service-broker/mgmtapi"
)

func (b *Broker) redeploy(instanceID, planID string, lastOperation OperationType, requestParams map[string]interface{}, previousPlanID *string) (int, error) {
	deploymentName := deploymentName(instanceID)
	oldManifest, err := b.boshClient.GetManifestRaw(deploymentName)
	if err != nil {
		return 0, fmt.Errorf("error getting old manifest: %s", err)
	}
	if oldManifest == nil {
		return 0, mgmtapi.NewDeploymentNotFoundError(fmt.Errorf("there is no old manifest, the instance must not exist"))
	}
	boshTaskID, _, err := b.deployer.Deploy(instanceID, planID, deploymentName, lastOperation, requestParams, oldManifest, previousPlanID)
	if err != nil {
		return 0, err
	}

	return boshTaskID, nil
}

const InstancePrefix = "service-instance_"

func deploymentName(instanceID string) string {
	return InstancePrefix + instanceID
}
