package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
)

func (b *Broker) Unbind(instanceID, bindingID string, details brokerapi.UnbindDetails) error {
	errs := func(context string, err error) error {
		b.logger.Printf("error %s: %s", context, err)
		return convertToDisplayableError(err, b.logger)
	}

	vms, manifest, err := b.gatherBindingContext(instanceID)
	switch err.(type) {
	case boshclient.DeploymentNotFoundError:
		b.logger.Println(fmt.Errorf("error unbinding: instance %s, not found", instanceID))
		return brokerapi.ErrInstanceDoesNotExist
	case error:
		return errs("gathering unbinding info", err)
	}

	requestParams := map[string]interface{}{
		"plan_id":    details.PlanID,
		"service_id": details.ServiceID,
	}

	b.logger.Printf("service adapter will delete binding with ID %s for instance %s\n", bindingID, instanceID)
	if err := b.serviceAdapter.DeleteBinding(bindingID, vms, manifest, requestParams); err != nil {
		return errs("deleting binding", err)
	}

	return nil
}
