package broker

import (
	"fmt"

	"github.com/pivotal-cf/on-demand-service-broker/mgmtapi"
)

func (b *Broker) Upgrade(instanceID string) (int, error) {
	b.deploymentLock.Lock()
	defer b.deploymentLock.Unlock()

	instance, err := b.cfClient.GetInstanceState(instanceID)
	if err != nil {
		if IsNotFound(err) {
			return 0, mgmtapi.NewDeploymentNotFoundError(err)
		}
		return 0, err
	}

	if instance.OperationInProgress {
		return 0, mgmtapi.NewOperationInProgressError(fmt.Errorf("cloud controller: operation in progress for instance %s", instanceID))
	}

	b.logger.Printf("upgrading instance %s", instanceID)

	taskID, err := b.redeploy(instanceID, instance.PlanID, OperationTypeUpgrade, nil, &instance.PlanID)
	if err != nil {
		b.logger.Printf("error upgrading instance %s: %s", instanceID, err)
		return 0, err
	}

	return taskID, nil
}
